notes = {
        "2022-07-15": "compressed imidazole smarts; split amines into primary, secondary, tertiary; change carboxylic acid value to ACDlab prediction; added monoand di-halogenated phenols",
        "2022-07-11": "added smarts patters to match tetrazole, substituted phenol, substituted imidazoles, pyridines. Use pKa values predicted by ACDlabs GALAS method; fixed most of smarts patters such that they explicit hydrogens compressed to heavy atom definition, otherwise match did not work; removed very generic definitions, like, aromatic N deprotonated as they hit too many unwanted structures",
        "2020-09-11": "added last column with acid-base classification"
    }